
# q-expansion of newform 350.2.e.l, downloaded from the LMFDB on 13 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 350
weight = 2
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[127, [1, 0]], [101, [0, -1]]]
aps_data = [[0, 1], [3, -3], [0, 0], [1, -3], [2, -2], [0, 0], [-4, 4], [0, 6], [0, 3], [9, 0], [4, -4], [0, -4], [-7, 0], [5, 0], [0, 8], [-2, 2], [-10, 10], [0, -1], [-9, 9], [2, 0], [-4, 4], [0, -10], [7, 0], [0, -1], [-14, 0], [-3, 3], [0, 1], [0, 3], [9, -9], [-2, 0], [-16, 0], [0, -8], [12, -12], [-14, 0], [0, -3], [16, -16], [10, -10], [0, -4], [21, 0], [0, 8], [-12, 12], [7, 0], [0, 18], [26, -26], [-2, 0], [4, -4], [-26, 0], [-28, 0], [-4, 4], [0, -22], [0, 24], [16, 0], [-10, 10], [0, 0], [0, 8], [5, -5], [-3, 3], [0, 6], [12, -12], [2, 0], [-4, 4], [-28, 0], [-7, 0], [18, -18], [0, 8], [0, -32], [0, 32], [26, 0], [19, -19], [-35, 0], [-18, 18], [0, 4], [-11, 11], [0, -4], [30, 0], [0, 15], [-26, 26], [0, 22], [0, -31], [-3, 3], [0, 0], [-19, 0], [30, -30], [-14, 0], [0, 20], [0, 31], [-33, 0], [0, -32], [-14, 0], [19, 0], [0, -13], [-24, 24], [-16, 16], [-12, 0], [0, 18], [21, 0], [0, -1], [-38, 38], [0, -20], [0, -3], [33, 0], [-2, 2], [17, -17], [0, 18], [30, -30], [10, -10], [28, 0], [0, -6], [-12, 12], [42, 0], [0, 1], [12, -12], [-44, 0], [46, -46], [2, 0], [-5, 5], [-28, 0], [-11, 11], [0, -4], [-26, 0], [11, -11], [-16, 0], [0, -48], [-37, 37], [0, -22], [-47, 0], [0, 11], [0, 6], [21, 0], [0, 22], [2, -2], [-9, 0], [0, 4], [-16, 0], [0, 6], [-14, 0], [24, -24], [31, -31], [0, 0], [51, -51], [-28, 0], [0, 18], [19, -19], [19, 0], [46, -46], [14, 0], [-14, 0], [-18, 18], [0, 48], [0, -11], [0, 38], [-7, 0], [12, 0], [0, 29], [5, -5], [30, 0], [0, -38], [0, -43], [28, 0], [46, -46], [0, -25], [12, 0], [-37, 0], [0, 48], [-30, 30], [17, -17], [-40, 40], [-46, 46]]
